_ = require 'underscore'

log4js = require 'log4js'
logger = log4js.getLogger 'RedisInterface'

redis = require 'redis'

BaseInterface = require './BaseInterface.coffee'

Request = require './Request.coffee'
Response = require './Response.coffee'

class RedisInterface extends BaseInterface
  constructor: (options = {}) ->
    logger.info 'initializing...'
    super

    @pubClient = redis.createClient options
    @subClient = redis.createClient options 
    
    @subscribtions = {}
    
    do @initListeners

RedisInterface::initListeners = ->
  @pubClient.on 'connect', (data) =>
    @pubClientConnected = true
    @emitter.emit 'pub.connect', data
    do @_connectHandler
  @pubClient.on 'ready', (data) => @emitter.emit 'pub.ready', data
  # @pubClient.on 'error', (data) => @emitter.emit 'pub.error', data

  @subClient.on 'connect', (data) =>
    @subClientConnected = true
    @emitter.emit 'sub.connect', data
    do @_connectHandler
  @subClient.on 'ready', (data) => @emitter.emit 'sub.ready', data
  # @subClient.on 'error', (data) => @emitter.emit 'sub.error', data

  @subClient.on 'subscribe', (data) => @emitter.emit 'subscribe', data
  @subClient.on 'message', (channel, message) =>
    @emitter.emit 'message',
      channel: channel
      message: message
    @emitter.emit "channel.#{channel}.message", message

RedisInterface::_connectHandler = ->
  if @pubClientConnected && @subClientConnected
    @emitter.emit 'connect'

RedisInterface::request = (requestChannel, data) ->
  req = new Request data
  new Promise (resolve, reject) =>
    subHandler = 
    @subscribe req.responseChannel, (message) =>
      resp = Response.parse message, req
      if resp then _handler resp
    @publish requestChannel, req.toJson()
    setTimeout =>
      _handler new Response req.key,
        message: 'Timeout'
      , 408
    , req.timeout

    _handler = _.once (resp) =>
      @unsubscribe req.responseChannel
      if resp.success
        return resolve resp
      if !resp.success
        return reject resp

RedisInterface::get = (channel, callback) ->
  @subscribe channel, (message) =>
    req = Request.parse message
    unless req then return
    callback req.data
    .then (res) =>
      Promise.resolve new Response req.key, res.data || null, res.status || 200
    , (err) =>
      Promise.resolve new Response req.key, err.data || 'Error', err.status || 500
    .then (resp) =>
      @publish req.responseChannel, resp.toJson()


RedisInterface::subscribe = (channel, callback = ->) ->
  @emitter.on "channel.#{channel}.message", (data) =>
    callback data
  unless @subscribtions[channel]
    @subClient.subscribe channel
    @subscribtions[channel] = true
  Promise.resolve()

RedisInterface::unsubscribe = (channel) ->
  @emitter.removeAllListeners "channel.#{channel}.message"
  @subClient.unsubscribe channel
  delete @subscribtions[channel]

RedisInterface::publish = (channel, data) ->
  if typeof data == 'object'
    try
      data = JSON.stringify data
    catch e
      return Promise.reject e
  @pubClient.publish channel, data
  Promise.resolve()
module.exports = RedisInterface