EventEmitter = require 'events'


class TaskEmitter extends EventEmitter

class Task
  constructor: (@id = @generateKey(), @action = null, @options = {}, @state = 'created', @result = null, @error = null, @createdAt = new Date) ->
    @emitter = new TaskEmitter
    do @initListeners

    @updatedAt = new Date

Task::initListeners = ->
  @emitter.on 'start', =>
    @updatedAt = new Date
    @state = 'started'
    
  # @emitter.on 'step', () =>

Task::generateKey = ->
  Math.floor(Math.random()*1000*1000*1000*1000*1000*1000).toString(32) +  Math.floor(Math.random()*1000*1000*1000*1000*1000*1000).toString(32)

Task::run = (fn) ->
  @emitter.emit 'start', @
  Promise.resolve()
  .then =>
    fn @
  .then (res) =>
    @updatedAt = new Date
    @state = 'completed'
    @result = res
    @emitter.emit 'complete', res
    Promise.resolve res
  , (err) =>
    @updatedAt = new Date
    @state = 'failed'
    @error = err
    @emitter.emit 'error', err
    Promise.reject err

Task::info = ->
  ret = 
    taskId: @id
    action: @action
    options: @options
    result: @result
    error: @error
    state: @state
    createdAt: @createdAt
    updatedAt: @updatedAt
  if @error then ret.error = 
    message: @error.message
  ret

Task.parse = (data) ->
  if typeof data == 'string'
    try
      data = JSON.parse data
    catch e
      return null
  if data && typeof data.taskId == 'string' && typeof data.action == 'string' && typeof data.options == 'object' && typeof data.state == 'string'
    return new Task data.taskId, data.action, data.options, data.state, data.result, data.error, data.createdAt
  return null

module.exports = Task